"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const select = require('unist-util-select');
const path_1 = __importDefault(require("path"));
const is_relative_url_1 = __importDefault(require("is-relative-url"));
const slash_1 = __importDefault(require("slash"));
const gatsby_plugin_ffmpeg_1 = require("gatsby-plugin-ffmpeg");
const allowedFiletypes = ['avi', 'mp4', 'mov', 'mkv'];
const defaults = {
    pipelines: [
        {
            name: 'vp9',
            transcode: (chain) => chain
                .videoCodec('libvpx-vp9')
                .noAudio()
                .outputOptions(['-crf 20', '-b:v 0']),
            maxHeight: 480,
            maxWidth: 900,
            fileExtension: 'webm',
        },
        {
            name: 'h264',
            transcode: (chain) => chain.videoCodec('libx264').noAudio(),
            maxHeight: 480,
            maxWidth: 900,
            fileExtension: 'mp4',
        },
    ],
};
module.exports = async function transform({ files, markdownNode, markdownAST, pathPrefix, getNode, reporter, }, pluginOptions) {
    const options = Object.assign({}, defaults, pluginOptions);
    // This will only work for markdown syntax image tags
    const markdownVideoNodes = select(markdownAST, `image`);
    // Takes a node and generates the needed videos and then returns
    // the needed HTML replacement for the video
    const generateVideosAndUpdateNode = async function (node) {
        // Check if this markdownNode has a File parent. This plugin
        // won't work if the video isn't hosted locally.
        const parentNode = getNode(markdownNode.parent);
        let videoPath;
        if (parentNode && parentNode.dir) {
            videoPath = (0, slash_1.default)(path_1.default.join(parentNode.dir, node.url));
        }
        else {
            return null;
        }
        const videoNode = files.find((file) => {
            if (file && file.absolutePath) {
                return file.absolutePath === videoPath;
            }
            return null;
        });
        if (!videoNode || !videoNode.absolutePath) {
            return null;
        }
        let transcodeResult = await (0, gatsby_plugin_ffmpeg_1.transcode)({
            file: videoNode,
            options,
            reporter,
        });
        // Calculate the paddingBottom %
        const sourceTags = transcodeResult.videos.map((video) => {
            return `<source src="${video.src}" type="video/${video.fileExtension}">`;
        });
        /*
        console.log(
          transcodeResult.presentationMaxWidth,
          transcodeResult.presentationMaxHeight
        );
        */
        let wrapperAspectStyle;
        let videoAspectStyle;
        if (transcodeResult.aspectRatio < 1) {
            wrapperAspectStyle = `max-width: ${transcodeResult.presentationMaxWidth}px; max-height: ${transcodeResult.presentationMaxHeight}px; margin-left: auto; margin-right: auto;`;
            videoAspectStyle = `height: 100%; width: 100%; margin: 0 auto; display: block; max-height: ${transcodeResult.presentationMaxHeight}px;`;
        }
        else {
            // we're landscape, use the video aspect ratio to create a
            const ratio = `${(1 / transcodeResult.aspectRatio) * 100}%`;
            wrapperAspectStyle = `position: relative; display: block; padding-top: ${ratio};`;
            videoAspectStyle = `position: absolute; top: 0; left: 0; width: 100%; height: auto;`;
        }
        const videoTag = `
    <video preload autoplay muted loop playsinline style="${videoAspectStyle}">
      ${sourceTags.join('')}
    </video>
    `;
        let rawHTML = `
      <div
      class="gatsby-video-aspect-ratio"
      style="${wrapperAspectStyle}"
      >${videoTag}</div>
    `;
        return rawHTML;
    };
    await Promise.all(markdownVideoNodes.map(async (node) => {
        const fileType = node.url.split('.').pop();
        if ((0, is_relative_url_1.default)(node.url) && allowedFiletypes.includes(fileType)) {
            const rawHTML = await generateVideosAndUpdateNode(node);
            if (rawHTML) {
                // Replace the video node with an inline HTML node.
                node.type = `html`;
                node.value = rawHTML;
            }
            return node;
        }
        else {
            // Video isn't relative so there's nothing for us to do.
            return null;
        }
    }));
};
//# sourceMappingURL=index.js.map